# -*- coding: utf-8 -*-

package = "pytrovich"
version = "0.0.1post2"
authors = ["Anton Alekseev"]
authors_string = ", ".join(authors)
emails = ["anton.m.alexeyev@gmail.com"]
license = "MIT"
copyright = "2020 " + authors_string
